@@ -14,7 +14,7 @@ class TextListField(Field):
   :param name: (str) A name that will be used to refer to the object as a variable and in the HTML form.
   :param label: (str) A human readable label for the field for the HTML form
   :param description: (Optional[str]) A long human readable description for the field for the HTML form
-  :param choices: (Union[List[str], Set[str], Dict[str, str]]) A set of choices that are available for this field or lookup table mapping from choice label to resulting value
+  :param choices: (Union[List[str], Dict[str, str]]) A set of choices that are available for this field or lookup table mapping from choice label to resulting value
   :param constraint: (Regex[str]) A regular expression for validating the file name.
   :param required: (Optional[bool]) Whether or not this field is required (defaults to false)
   :param default: (List[str]) A default value as an example and for use during prototyping
@@ -71,3 +71,45 @@ class TextListField(Field):
       return [self.choices[v] for v in self.raw_value]
     else:
       return self.raw_value
+
+  def to_jsonschema(self):
+    schema = {'type': 'array', 'items': { 'type': 'string' } }
+    if self.args.get('label'): schema['title'] = self.args['label']
+    if self.args.get('description'): schema['description'] = self.args['description']
+    if self.args.get('choices'): schema['items']['enum'] = list(self.args['choices'])
+    if self.args.get('constraint'): schema['items']['pattern'] = re_full(self.args['constraint'])
+    if self.args.get('default'): schema['default'] = self.args['default']
+    return schema
+
+  def to_cwl(self):
+    schema = super().to_cwl()
+    # NOTE: CWL's array enum is broken upstream
+    # if self.args.get('choices'):
+    #   if self.args.get('required') == True:
+    #     schema['type'] = {
+    #       'type': 'array',
+    #       'items': {
+    #         'type': 'enum',
+    #         'symbols': list(self.args['choices'])
+    #       },
+    #     }
+    #   else:
+    #     schema['type'] = ['null', {
+    #       'type': 'array',
+    #       'items': {
+    #         'type': 'enum',
+    #         'symbols': list(self.args['choices'])
+    #       },
+    #     }]
+    schema['type'] = f"string{'' if self.args.get('required') == True else '?'}"
+    return schema
+
+  def to_cwl_value(self):
+    return '\n'.join(self.raw_value)
+
+  def to_click(self):
+    args, kwargs = super().to_click()
+    # kwargs['multiple'] = True
+    import click
+    kwargs['type'] = click.STRING
+    return args, kwargs
